/*---------------------------------------
   EZFONT.C -- Easy Font Creation
               (c) Charles Petzold, 1996
  ---------------------------------------*/

#include <windows.h>
#include <math.h>
#include <errno.h>

#include "ezfont.h"

#ifndef _MAX_PATH
#define _MAX_PATH 260
#endif

extern int cxClient ;
extern int cyClient ;

//*************************************************************************
HFONT build_font(char *fname, unsigned fheight, unsigned fbold, unsigned fitalic, unsigned funderline, unsigned fstrikeout)
{
   HFONT hfont = CreateFont(
         //  46, 28, 215, 0,  //  height, width, escapement, orientation
         fheight, 0, 0, 0,
         (fbold == 0) ? FW_NORMAL : FW_BOLD,
         fitalic,
         funderline,
         fstrikeout,
         0, 0, 0, 0, 0, //  other stuff
         fname);
   return hfont;
}

//*************************************************************************
static HFONT EzCreateFont (HDC hdc, char * szFaceName, int iDeciPtHeight,
       int iDeciPtWidth, unsigned iAttributes, int textangle, BOOL fLogRes)
{
   FLOAT      cxDpi, cyDpi ;
   HFONT      hFont ;
   LOGFONT    lf ;
   POINT      pt ;
   TEXTMETRIC tm ;

   SaveDC (hdc) ;

   SetGraphicsMode (hdc, GM_ADVANCED) ;
   ModifyWorldTransform (hdc, NULL, MWT_IDENTITY) ;
   SetViewportOrgEx (hdc, 0, 0, NULL) ;
   SetWindowOrgEx   (hdc, 0, 0, NULL) ;

   if (fLogRes) {
      cxDpi = (FLOAT) GetDeviceCaps (hdc, LOGPIXELSX) ;
      cyDpi = (FLOAT) GetDeviceCaps (hdc, LOGPIXELSY) ;
   }
   else {
      cxDpi = (FLOAT) (25.4 * GetDeviceCaps (hdc, HORZRES) /
                              GetDeviceCaps (hdc, HORZSIZE)) ;

      cyDpi = (FLOAT) (25.4 * GetDeviceCaps (hdc, VERTRES) /
                              GetDeviceCaps (hdc, VERTSIZE)) ;
   }

   pt.x = (long) (iDeciPtWidth  * cxDpi / 72) ;
   pt.y = (long) (iDeciPtHeight * cyDpi / 72) ;

   DPtoLP (hdc, &pt, 1) ;

   lf.lfHeight         = - (int) (fabs ((double) pt.y) / 10.0 + 0.5) ;
   lf.lfWidth          = 0 ;
   lf.lfEscapement     = textangle ;
   lf.lfOrientation    = textangle ;
   lf.lfWeight         = iAttributes & EZ_ATTR_BOLD      ? 700 : 0 ;
   lf.lfItalic         = iAttributes & EZ_ATTR_ITALIC    ?   1 : 0 ;
   lf.lfUnderline      = iAttributes & EZ_ATTR_UNDERLINE ?   1 : 0 ;
   lf.lfStrikeOut      = iAttributes & EZ_ATTR_STRIKEOUT ?   1 : 0 ;
   lf.lfCharSet        = 0 ;
   lf.lfOutPrecision   = 0 ;
   lf.lfClipPrecision  = 0 ;
   lf.lfQuality        = 0 ;
   lf.lfPitchAndFamily = 0 ;
   strcpy (lf.lfFaceName, szFaceName) ;
   hFont = CreateFontIndirect (&lf) ;

   if (iDeciPtWidth != 0) {
      hFont = (HFONT) SelectObject (hdc, hFont) ;

      GetTextMetrics (hdc, &tm) ;

      DeleteObject (SelectObject (hdc, hFont)) ;

      lf.lfWidth = (int) (tm.tmAveCharWidth *
                          fabs ((double) pt.x) / fabs ((double) pt.y) + 0.5) ;

      hFont = CreateFontIndirect (&lf) ;
   }
   RestoreDC (hdc, -1) ;
   return hFont ;
}

//*************************************************************************
static char ezfontname[_MAX_PATH] = "" ;
static int eztextangle = 0 ;  //  in tenths-of-a-degree
static int ezattribs = 0 ;
#define  DEFAULT_PSIZE  (200)
static int ezpointsize = DEFAULT_PSIZE ;

//****************************************************************************
int textheight(HWND hwnd)
{
   int diffY ;
   TEXTMETRIC tm ;
   HFONT hfont ;
   HDC hdc = GetDC (hwnd) ;

   if (ezfontname[0] != 0) {
      hfont = EzCreateFont (hdc, ezfontname, ezpointsize, 0, ezattribs, eztextangle, TRUE) ;
      SelectObject (hdc, hfont) ;
   }
   // SelectObject (hdc, CrDataFont) ;
   GetTextMetrics(hdc, &tm) ;
   diffY = tm.tmHeight + tm.tmExternalLeading ;
   // DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT)));
   if (ezfontname[0] != 0)
      DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT)));
   ReleaseDC (hwnd, hdc) ;
   return diffY ;
}

//*****************************************************************
int textheight(HDC hdc, unsigned selected)
{
   int diffY ;
   TEXTMETRIC tm ;
   HFONT hfont ;

   if (!selected  &&  ezfontname[0] != 0) {
      hfont = EzCreateFont (hdc, ezfontname, ezpointsize, 0, ezattribs, eztextangle, TRUE) ;
      SelectObject (hdc, hfont) ;
   }
   GetTextMetrics(hdc, &tm) ;
   diffY = tm.tmHeight + tm.tmExternalLeading ;
   if (!selected  &&  ezfontname[0] != 0) 
      DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT)));
   return diffY ;
}

//*****************************************************************
int textwidth(HDC hdc, char *str)
{
   HFONT hfont ;

   if (ezfontname[0] != 0) {
      hfont = EzCreateFont (hdc, ezfontname, ezpointsize, 0, ezattribs, eztextangle, TRUE) ;
      SelectObject (hdc, hfont) ;
   }
   SIZE ssize ;
   GetTextExtentPoint32(hdc, str, strlen(str), &ssize) ;
   if (ezfontname[0] != 0)
      DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT)));
   return ssize.cx ;
}

//****************************************************************************
void dprints(HDC hdc, int x, int y, unsigned attr, char *str)
{
   HFONT hfont ;

   if (ezfontname[0] != 0) {
      hfont = EzCreateFont (hdc, ezfontname, ezpointsize, 0, ezattribs, eztextangle, TRUE) ;
      SelectObject (hdc, hfont) ;
   }
   SetBkMode(hdc, TRANSPARENT) ;
   SetTextColor(hdc, attr) ;
   TextOut (hdc, x, y, str, strlen(str));
   SetBkMode(hdc, OPAQUE) ;
   if (ezfontname[0] != 0)
      DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT)));
}         

//****************************************************************************
void dprints(HDC hdc, int x, int y, unsigned fg, unsigned bg, char *str)
{
   HFONT hfont ;

   if (ezfontname[0] != 0) {
      hfont = EzCreateFont (hdc, ezfontname, ezpointsize, 0, ezattribs, eztextangle, TRUE) ;
      SelectObject (hdc, hfont) ;
   }
   SetTextColor(hdc, fg) ;
   SetBkColor  (hdc, bg) ;
   TextOut (hdc, x, y, str, strlen(str));
   if (ezfontname[0] != 0)
      DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT)));
}         

//*****************************************************************
void dprints_centered_x(HDC hdc, long y, unsigned attr, char *str)
{
   SIZE ssize ;
   long xt ;
   HFONT hfont ;

   if (ezfontname[0] != 0) {
      hfont = EzCreateFont (hdc, ezfontname, ezpointsize, 0, ezattribs, eztextangle, TRUE) ;
      SelectObject (hdc, hfont) ;
   }
   GetTextExtentPoint32(hdc, str, strlen(str), &ssize) ;
   xt = (cxClient - ssize.cx) / 2 ;
   dprints(hdc, xt, y-(ssize.cy/2), attr, str) ;
   if (ezfontname[0] != 0)
      DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT)));
}

//*****************************************************************
void dprints_centered_x(HDC hdc, long y, unsigned fg, unsigned bg, char *str)
{
   SIZE ssize ;
   long xt ;
   HFONT hfont ;

   if (ezfontname[0] != 0) {
      hfont = EzCreateFont (hdc, ezfontname, ezpointsize, 0, ezattribs, eztextangle, TRUE) ;
      SelectObject (hdc, hfont) ;
   }
   GetTextExtentPoint32(hdc, str, strlen(str), &ssize) ;
   xt = (cxClient - ssize.cx) / 2 ;
   dprints(hdc, xt, y-(ssize.cy/2), fg, bg, str) ;
   if (ezfontname[0] != 0)
      DeleteObject (SelectObject (hdc, GetStockObject (SYSTEM_FONT)));
}

//*****************************************************************
void dprints_centered_x(HWND hwnd, long y, unsigned attr, char *str)
{
   HDC hdc = GetDC (hwnd) ;
   dprints_centered_x(hdc, y, attr, str) ;
   ReleaseDC (hwnd, hdc) ;
}

//*****************************************************************
void dprints_centered_x(HWND hwnd, long y, unsigned fg, unsigned bg, char *str)
{
   HDC hdc = GetDC (hwnd) ;
   dprints_centered_x(hdc, y, fg, bg, str) ;
   ReleaseDC (hwnd, hdc) ;
}

